<?php

namespace Drupal\ckeditor_special_symbols\Plugin\CKEditorPlugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginContextualInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Entity\Editor;

/**
 * Defines the "CKEditor Special Symbols" plugin.
 *
 * @CKEditorPlugin(
 *   id = "jsplus_special_symbols",
 *   label = @Translation("CKEditor Special Symbols"),
 *   module = "ckeditor_special_symbols"
 * )
 */
class ckeditor_special_symbols extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginContextualInterface {

  public $plugins = array(
     'jsplus_special_symbols' => array(
    'urlDoc' => 'http://js.plus/products/special-symbols/configuration',
    'buttons' => array(array('label' => 'Insert special symbol')),
    'params' => array(
        array(
            'name' => 'jsplus_special_symbols',
            'default' => '0x0020-0x002F,x00A0-x02AF,4567-4569,a,b,c',
            'type' => 'str',
            'order' => 1000,
            'title' => 'Format of symbols',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_special_symbols_use_dialog',
            'default' => FALSE,
            'type' => 'bool',
            'order' => 1010,
            'title' => 'Using of dialog by default',
            'hint' => 'You can use dialog or popup with these option',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_special_symbols_popup_width',
            'default' => 400,
            'type' => 'int',
            'order' => 1020,
            'title' => 'Width of popup',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_special_symbols_font_size',
            'default' => 16,
            'type' => 'int',
            'order' => 1030,
            'title' => 'Set size of symbols inside plugin\'s dropdown menu or dialog.',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_special_symbols_images_allowed',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1040,
            'title' => 'Is image generation is allowed',
            'hint' => '',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_special_symbols_images_by_default',
            'default' => FALSE,
            'type' => 'bool',
            'order' => 1050,
            'title' => 'Is image generation set by default.',
            'hint' => '',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_special_symbols_images_font_default',
            'default' => 'Arial',
            'type' => 'str',
            'order' => 1060,
            'title' => 'Default font family for icons',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_special_symbols_images_font_size_default',
            'default' => 12,
            'type' => 'int',
            'order' => 1070,
            'title' => 'Default size for font icons',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_special_symbols_images_generator_url',
            'default' => '',
            'type' => 'str',
            'order' => 1080,
            'title' => 'Url of image generator',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_special_symbols_images_fonts',
            'default' => '',
            'type' => 'str',
            'order' => 1090,
            'title' => 'Fonts available for image generation',
            'hint' => '',
            'widget' => 'text'
        ),
    )
)
  );

    /**
     * {@inheritdoc}
     */
    public function getButtons() {
        if (!$this->isInstalled())
            return array();

        $buttons = array();
        foreach ($this->plugins as $pluginName => $pluginDef) {

            if (isset($pluginDef['buttons'])) {
                foreach($pluginDef['buttons'] as $buttonName => $buttonDef) {
                    $image = 'https://cdn.n1ed.com/cdn/buttons/' . $pluginName . '.png';
                    $button = array(
                        'label' => $buttonDef['label'],
                        'image' => $image
                    );
                    $buttons[$pluginName] = $button;
                }
            }

        }
        return $buttons;
    }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    return '/libraries/jsplus_special_symbols/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function isInternal() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled(Editor $editor) {
    return TRUE;
  }

  function getConfigParam($settings, $param, $default, $type, $inside) {
    $name = $param;
    if (isset($settings[$name]) && strlen($settings[$name]) > 0)
        $value = $settings[$name];
    else
        $value = $default;
    if (isset($type) && $type == 'int') {
        $value = intval($value);
    } else if (isset($type) && $type == 'bool') {
        if ($value == '1')
            $value = true;
        else if ($value == '0')
            $value = false;
    } else if (isset($type) && $type == 'json') {
        $value = json_decode($value);
    }
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
      $settings = array();
      if (isset($editor->getSettings()['plugins']['jsplus_special_symbols']))
        $settings = $editor->getSettings()['plugins']['jsplus_special_symbols'];
      $result = array();
      foreach ($this->plugins as $pluginName => $pluginDef) {
          if (isset($pluginDef['params'])) {
              foreach ($pluginDef['params'] as $paramDef) {
                  $value = $this->getConfigParam($settings, $paramDef['name'], $paramDef['default'], $paramDef['type'], isset($paramDef['inside']) ? $paramDef['inside'] : null);
                  if (!array_key_exists('inside', $paramDef))
                      $result[$paramDef['name']] = $value;
                  else {
                      if (!isset($result[$paramDef['inside']]))
                          $result[$paramDef['inside']] = array();
                      $result[$paramDef['inside']][$paramDef['name']] = $value;
                  }
              }
          }
      }
      return $result;
  }

  function addSelectToForm(& $form, $settings, $param, $title, $default, $options, $inside, $urlHelp) {
      $form[$param] = array(
        '#type' => 'select',
        '#title' => $title,
        '#options' => $options,
        '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
        '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
      );
  }

  function addTextboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textfield',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addTextareaToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textarea',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addCheckboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'checkbox',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'bool', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  public function isInstalled() {
    return file_exists($_SERVER['DOCUMENT_ROOT'] . '/libraries/jsplus_special_symbols');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $editor_settings = $editor->getSettings();
    if (isset($editor_settings['plugins']['jsplus_special_symbols']))
      $settings = $editor_settings['plugins']['jsplus_special_symbols'];

    $form['#attached']['library'][] = 'ckeditor_special_symbols/ckeditor_special_symbols.admin';

    if (!$this->isInstalled()) {
        $form['warning'] = array(
            '#markup' => 'Looks like CKEditor Special Symbols (Drupal 8 module) is installed but CKEditor add-ons not found. In order to use this module please copy CKEditor add-ons into "libraries" forder in the root of your Drupal 8 installation (create the directory if it does not exist).'
        );
        return $form;
    }

    $params = array();
    foreach ($this->plugins as $pluginName => $pluginDef) {
        if (isset($pluginDef['params']))
            foreach ($pluginDef['params'] as $paramDef) {
                $paramDef['urlDoc'] = $pluginDef['urlDoc'];
                $params[$paramDef['order']] = $paramDef;
            }
    }
    ksort($params); // sort by key (order)

    if (count($params) > 0) {
        foreach ($params as $order => $paramDef) {
            $inside = isset($paramDef['inside']) ? $paramDef['inside'] : null;
            if ($paramDef['type'] == 'json')
                $paramDef['title'] = $paramDef['title'] . "<br/>\nImportant notice: this value needs to be in JSON format. So this is the must to use double quotes (not single ones) for all key names and all string values";
            if ($paramDef['widget'] == 'select') {
                $this->addSelectToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['widgetOptions'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'checkbox') {
                $this->addCheckboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'textarea') {
                $this->addTextareaToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else {
                $this->addTextboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            }
        }
    } else {
        //$settings = array();
        $form['warning'] = array(
            '#markup' => 'This add-on does not have parameters.'
        );
    }

    return $form;
  }

}
